'
' Rename one or more files according to input and output templates
'
' Written by Keith Baines in HiSoft Basic.
' January 1990
'

DEFINT A-Z
LIBRARY "GEMDOS"


DIM SHARED DTA(21)				' Buffer for fsfirst/fsnext calls

COMMON SHARED NameChars$,CMD$

CMD$=COMMAND$

NameChars$="ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_!@#$%^&()+-=~`;'"",<>|[]{}"

CONST TRUE=-1
CONST FALSE=0

CALL Banner

IF CMD$="" THEN
   Usage
   PRINT " Enter Command (or [Return] to quit):"
   INPUT " MREN ",CMD$
   CMD$=UCASE$(CMD$)
   IF CMD$="" THEN STOP -1
   CLS
   CALL Banner
END IF

PRINT "MREN "+CMD$
PRINT

CommandParse
   
GetTemplates Path$,F1$,F2$

IF FNCheckPath(Path$)=0 THEN
   PRINT "ERROR: Invalid Path: ";_
          LEFT$(FNGetToken$(0),FNRightIndex(FNGetToken$(0),"\"))
   STOP
ELSEIF FNCheckInfile(F1$)=0 THEN
   PRINT "ERROR: Invalid Input Template: ";_
          MID$(FNGetToken$(0),FNRightIndex(FNGetToken$(0),"\")+1)
   STOP
ELSEIF FNCheckOutfile(F2$)=0 THEN
   PRINT "ERROR: Invalid Output Template: ";FNGetToken$(1)
   STOP
ELSEIF FNCheckOption(FNGetToken$(2))=0 THEN
   PRINT "ERROR: Invalid Option ";FNGetToken$(2)
   STOP
END IF



CALL Init_DTA
Get_Files Path$+F1$
IF Num_Files<=0 THEN
   PRINT "No Files to rename."
ELSE
   Rename_Files Path$,F2$
END IF
Reset_DTA
STOP


'
' Check for valid option flag
'

DEF FNCheckOption(A$)
   FNCheckOption = (A$="" OR A$=="A" OR A$=="D" OR A$=="R")
END DEF

'
' Check for valid file path
'
DEF FNCheckPath(P$)
   LOCAL I,D$,F$
   I=INSTR(P$,":")
   IF I=2 THEN
      D$=LEFT$(P$,1)
      IF D$<"A" OR D$>"P" THEN
         FNCheckPath=0
         EXIT DEF
      ELSE
         P$=MID$(P$,3)
      END IF
   ELSEIF I<>0 THEN
      FNCheckPath=0
      EXIT DEF
   END IF

   IF LEFT$(P$,1)="\" THEN
      P$=MID$(P$,2)
   END IF
      
   DO WHILE LEN(P$)>0
      I=INSTR(P$,"\")
      IF I=0 THEN
         F$=P$
         P$=""
      ELSEIF I=1 THEN
         FNCheckPath=0
         EXIT DEF
      ELSE
         F$=LEFT$(P$,I-1)
         P$=MID$(P$,I+1)
      END IF
      
      IF FNCheckFileName(F$,NameChars$)=0 THEN
         FNCheckPath=0
         EXIT DEF
      END IF
   LOOP
   
   FNCheckPath=1
END DEF

'
' Check for valid input file template
'
DEF FNCheckInfile(F$)
   FNCheckInfile=FNCheckFileName(F$,NameChars$+"?")
END DEF

'
' Check for valid output file template
'
DEF FNCheckOutfile(F$)
   FNCheckOutfile=FNCheckFileName(F$,_
    NameChars$+"?"+CHR$(1)+CHR$(2)+CHR$(3)+CHR$(4)+_
                   CHR$(5)+CHR$(6)+CHR$(7)+CHR$(8)+_
                   CHR$(10)+CHR$(11)+CHR$(12))
END DEF

'
' Check a file template against valid characters in V$
'
DEF FNCheckFileName(F$,V$)
   LOCAL I,N$,T$

   I=INSTR(F$,".")
   IF I=0 THEN
      N$=F$
      T$=""
   ELSEIF I=1 THEN
      FNCheckFileName=0
      EXIT DEF
   ELSE
      N$=LEFT$(F$,I-1)
      T$=MID$(F$,I+1)
   END IF
   IF LEN(N$)>8 OR LEN(T$)>3 THEN
      FNCheckFileName=0
      EXIT DEF
   END IF
   IF FNCheckString(N$,V$)=0 _
   OR FNCheckString(T$,V$)=0 THEN
      FNCheckFileName=0
   ELSE
      FNCheckFileName=1
   END IF

END DEF

'
' Check that all characters in S$ are from V$
'
DEF FNCheckString(S$,V$)
   LOCAL L,I
   L=LEN(S$)
   FOR I=1 TO L
      IF INSTR(V$,MID$(S$,I,1))=0 THEN
         FNCheckString=0
         EXIT DEF
      END IF
   NEXT I
   FNCheckString=1
END DEF
      
      
'
' Set the DTA to our internal buffer
'
SUB Init_DTA
   SHARED Old_DTA&,Our_DTA&
   Old_Dta&=FNfgetdta&
   Our_DTA&=VARPTR(DTA(0))
   fsetdta Our_DTA&
END SUB

'
' Restore GEMDOS's original DTA address
'
SUB Reset_DTA
   SHARED Old_DTA&
   fsetdta Old_DTA&
END SUB


'
' Get the Ith command line token
'
DEF FNGetToken$(I)
   SHARED C__Tokens$(),N__Tokens
   IF I>N__Tokens THEN
      FNGetToken$=""
   ELSE
      FNGetToken$=C__Tokens$(I)
   END IF
END DEF

'
' Parse the command line into blank or comma separated tokens
'
SUB CommandParse
   SHARED C__Tokens$(),N__Tokens
   LOCAL I,L,C$
   DIM C__Tokens$(10)
   
   L=LEN(CMD$)

   I=0
   N__Tokens=0
   C__Tokens$(0)=""
   
   FOR I=1 TO L
      C$=MID$(CMD$,I,1)
      IF C$="," THEN
         INCR N__Tokens
         C__Tokens$(N__Tokens)=""
      ELSEIF C$=" " THEN
         IF C__Tokens$(N__Tokens)<>"" THEN
            INCR N__Tokens
            C__Tokens$(N__Tokens)=""
         END IF
      ELSE
         C__Tokens$(N__Tokens)=C__Tokens$(N__Tokens)+C$
      END IF
      IF N__Tokens=10 THEN EXIT FOR
   NEXT I
   
END SUB
         
   
'
' Get the path and file templates from the command line
'
SUB GetTemplates(P$,F1$,F2$)
   LOCAL T$,C$,I

   P$=FNGetToken$(0)
   I=FNRightIndex(P$,"\")

   F1$=MID$(P$,I+1)
   P$=LEFT$(P$,I)

   F2$=FNGetToken$(1)
   
   FOR I=1 TO LEN(F2$)
      C$=MID$(F2$,I,1)
      IF C$="~" AND I<>LEN(F2$) THEN
         INCR I
         C$=MID$(F2$,I,1)
         IF C$>="1" AND C$<="8" THEN
            T$=T$+CHR$(ASC(C$)-ASC("0"))
         ELSEIF C$>="A" AND C$<="C" THEN
            T$=T$+CHR$(10+ASC(C$)-ASC("A"))
         ELSE
            T$=T$+C$
         END IF
      ELSE
         T$=T$+C$
      END IF
   NEXT I
   F2$=T$
   
   F1$=FNExpand$(F1$,1)
   F2$=FNExpand$(F2$,2)
END SUB
   

'
' Expand * in file name to correct number of ?s
'
DEF FNExpand$(F$,N)
   LOCAL I,N$,T$
   I=INSTR(F$,".")
   IF I=0 THEN
      N$=F$
      T$=""
   ELSE
      N$=LEFT$(F$,I-1)
      T$=MID$(F$,I+1)
   END IF
   
   DO
      I=INSTR(N$,"*")
      IF I=0 THEN EXIT LOOP
      N$=LEFT$(N$,I-1)+STRING$(9-LEN(N$),"?")+MID$(N$,I+1)
   LOOP
   
   DO
      I=INSTR(T$,"*")
      IF I=0 THEN EXIT LOOP
      T$=LEFT$(T$,I-1)+STRING$(4-LEN(T$),"?")+MID$(T$,I+1)
   LOOP

   
   IF N=2 THEN
      FOR I=1 TO LEN(N$)
         IF MID$(N$,I,1)="?" THEN
            MID$(N$,I,1)=CHR$(I)
         END IF
      NEXT I
      FOR I=1 TO LEN(T$)
         IF MID$(T$,I,1)="?" THEN
            MID$(T$,I,1)=CHR$(9+I)
         END IF
      NEXT I
   END IF
   FNExpand$=N$+"."+T$
   
END DEF


'
' Find all files matching input template
'
SUB Get_Files(Template$)
   SHARED Num_Files,Max_Files,Files$()
   LOCAL RC

   Max_Files=100
   Num_Files=0
   DIM Files$(Max_Files)

   RC=FNfsfirst(Template$,0)
   DO WHILE RC=0
      Files$(Num_Files)=FNGetName$
      INCR Num_Files
      IF Num_Files>Max_Files THEN
         Max_Files=Max_Files+100
         REDIM APPEND Files$(Max_Files)
      END IF
      RC=FNfsnext
   LOOP
END SUB


'
' Get file name from the DTA
'
DEF FNGetName$
   SHARED Our_DTA&
   LOCAL N$,P&,I,B
   P&=Our_DTA&+30
   N$=""
   FOR I=1 TO 14
      B=PEEKB(P&)
      IF B=0 THEN EXIT FOR
      N$=N$+CHR$(B)
      INCR P&
   NEXT I
   FNGetName$=N$
END DEF



SUB BANNER
   PRINT " MULTI-RENAME   --  a PD utility by Keith Baines  --  January 1990"
END SUB

SUB Usage
   PRINT
   PRINT " USAGE:    MREN [<Path>\]<Template1> <Template2> [A|D|R]"
   PRINT
   PRINT " Renames all files matching <Template1> according to <Template2>."
   PRINT
   PRINT " FLAGS:   A  Ask whether to rename each matching file."
   PRINT "          D  Don't rename: just list file names (this is the default)."
   PRINT "          R  Rename all matching files."
   PRINT
   PRINT " Template1: Usual TOS file template including ? and *"
   PRINT
   PRINT " Template2: ? copies corresponding character from template 1"
   PRINT "            * copies zero or more characters from template 1"
   PRINT "     ~1 to ~8 copy first to eighth file-name character from template 1"
   PRINT "     ~A ~B ~C copy first, second or third file type character from template 1"
   PRINT "            all other characters replace the input character at their position."
   PRINT
   PRINT " EXAMPLE    MREN B:\GDOS.SYS\AT????EP.FNT X1~3~4~5~6.*"
   PRINT " This will rename B:\GDOS.SYS\ATTR08EP.FNT as B:\GDOS.SYS\X1TR08.FNT"
   PRINT " and similarly for other matching files."
   PRINT
END SUB


'
' Do the re-naming
'
SUB Rename_Files(Path$,Template$)
   SHARED Num_Files,Files$()
   LOCAL I,F1$,F2$
   FOR I=0 TO Num_Files-1
      F1$=Files$(I)
      F2$=FNNewName$(Files$(I),Template$)
      PRINT " Rename ";F1$ TAB(22) " as ";F2$;
      F1$=Path$+F1$
      F2$=Path$+F2$
      IF FEXISTS(F2$) THEN
         PRINT TAB(40) " *** Already exists - can't rename"
      ELSEIF FNGetToken$(2)="R" THEN
         NAME F1$ AS F2$
         PRINT
      ELSEIF FNGetToken$(2)="A" THEN
         IF FNGetAnswer THEN
            NAME F1$ AS F2$
         END IF
      ELSE
         PRINT " (Just testing)"
      END IF
   NEXT I
END SUB


'
' Get a Y/N answer
'
DEF FNGetAnswer
   SHARED K$
   PRINT " ?  [Y/N]";
   LOCATE ,,1
   DO
      K$=UCASE$(INKEY$)
   LOOP UNTIL (K$="Y") OR (K$="N")
   LOCATE ,,0
   PRINT STRING$(5,CHR$(8));CHR$(27)+"K";K$
   FNGetAnswer=(K$="Y")
END DEF

'
' Construct new name given old name and output template
'
DEF FNNewName$(N$,T$)
   LOCAL X$,Y$,C$,I,J
   
   I=INSTR(N$,".")
   IF I=0 THEN
      N$=LEFT$(N$+SPACE$(8),8)+"."
   ELSEIF I<9 THEN
      N$=LEFT$(N$,I-1)+SPACE$(9-I)+MID$(N$,I)
   END IF
   N$=LEFT$(N$+"   ",12)
   
   X$=""
   J=0
   FOR I=1 TO LEN(T$)
      INCR J
      C$=MID$(T$,I,1)
      IF C$="?" THEN
         X$=X$+MID$(N$,J,1)
      ELSEIF C$<" " THEN
         X$=X$+MID$(N$,ASC(C$),1)
      ELSE
         X$=X$+C$
      END IF
   NEXT I
   Y$=""
   FOR I=1 TO LEN(X$)
      IF MID$(X$,I,1)<>" " THEN Y$=Y$+MID$(X$,I,1)
   NEXT I 
   
   FNNewName$=Y$
END DEF


'
' Find the position of the right-most occurence of s2$ in s1$
'   
DEF FNRightIndex(s1$,s2$)
STATIC i,j
   j=0
   DO
      i=j
      j=INSTR(i+1,s1$,s2$)
   LOOP UNTIL j=0
   FNRightIndex=i
END DEF
   