'
' Program to display available screen fonts.
' Each font is shown in its largest available (undoubled) size.
'
' by Keith Baines
'
' August 1991
'
' Converted to HiSoft BASIC 2 
' by John Watkins 3.Jan.94
' Added FontID 


REM $OPTION Y+,K10		' Leave lots of memory for loading fonts & drivers

DEFINT A-Z

LIBRARY "GEMVDI","GEMAES"


CALL MAIN
STOP -1



SUB MAIN

	LOCAL Button
	SHARED Printer,DUMMY
	
	MOUSE 0

	' Check GDOS is loaded.  If not, make a quick exit.
	IF NOT GDOS THEN
		DUMMY=form_alert(1,"[3][ | This Program Needs GDOS ][ QUIT ]")
		EXIT SUB
	END IF
	
	' Main loop over display/print/quit choices
	DO
		Button=form_alert(1,"[2][ |"+_
		" GDOS Font Display/Print |"+_
		"     by Keith Baines |"+_
		"       August 1991 | ]"+_
		"[SCREEN|PRINTER|QUIT]")
		IF Button=3 THEN
			EXIT SUB
		ELSE
			Printer = (Button=2)
			ShowFonts
		END IF
	LOOP
END SUB


SUB ShowFonts

	LOCAL FontName$,FontHeight,MaxPoint
	LOCAL F,P,Y,FontNum

	SHARED DUMMY,Printer
	SHARED PageLength,PageWidth,VSpacing,HSpacing,TopMargin,LeftMargin
	SHARED LastFont,FirstFont
	
	Initialise	
	
	Y=TopMargin
	
	FOR F=FirstFont TO LastFont
		FontNum=vqt_name(F,FontName$)		' Get font name & id number
		vst_font FontNum
		vst_point 1000								' Set largest size
		MaxPoint=PEEKW(intout)					' Get actual largest size

		FOR P=1 TO MaxPoint						' Try all font sizes up to max
			vst_point P
			IF PEEKW(intout)=P THEN				' If this size is available...
				FontHeight=PEEKW(ptsout+6)		' Get cell height
				IF Y+2*FontHeight>PageLength THEN	' Check if at bottom of page
					NewPage

					'v_updwk/v_clrwk (in NewPage) sometimes loose the clipping
					'rectangle and font face & size (a bug?), so re-set them.
					vs_clip 1,LeftMargin,0,PageWidth,PageLength
					vst_font FontNum
					vst_point P

					Y=TopMargin
				END IF
				showfont FontName$,P,Y,FontHeight,FontNum	' Display the font
				Y=Y+2*FontHeight+VSpacing			' Update vertical position
			END IF
		NEXT P
		Y=Y+2*VSpacing								' Add extra space between fonts
	NEXT F
	
	NewPage											' output last page

	ClearUp

END SUB


'
' Open printer workstation if necessary,
' calculate margins and text spacing
' and load fonts
'
SUB Initialise

	SHARED DUMMY,Printer,OldHandle
	SHARED PageLength,PageWidth,VSpacing,HSpacing,TopMargin,LeftMargin
	SHARED LastFont,FirstFont

	LOCAL inv(10),outv(57),I
	LOCAL NewHandle

	IF Printer THEN
		MOUSE 2										'busy-bee
		OldHandle=CVHANDLE						'save screen handle
		inv(0)=21									'printer id in assign.sys
		FOR I=1 TO 9: inv(I)=1: NEXT I
		inv(10)=2									'use raster co-ords
		v_opnwk inv(),NewHandle,outv()		'open the workstation
		IF NewHandle=0 THEN
			DUMMY=form_alert(1,"[3][ | Can't open printer ][ QUIT ]")
			STOP -1
		END IF
		PageLength=254E3/outv(4)				'10 inches converted to pixels
		VSpacing=6.4E3/outv(4)					'1/4 inch
		TopMargin=25.4E3/outv(4)				'1 inch
		PageWidth=177.8E3/outv(3)				'7 inches
		HSpacing=6.4E3/outv(3)					'1/4 inch
		LeftMargin=25.4E3/outv(3)				'1 inch
	ELSE	
		MOUSE -1										'mouse off
		WINDOW OPEN 2,"",0,0,0,0,0				'open window
		WINDOW FULLW								'and expand to full screen
		CLS
		WINDOW OFF									'program will do own clipping
		PageLength=PEEKW(SYSTAB+46)			'height of window in pixels
		VSpacing = PageLength\50
		TopMargin=PageLength\25
		PageWidth=PEEKW(SYSTAB+44)				'width of window in pixels
		HSpacing = PageWidth\25
		LeftMargin=PageWidth\25
		PageWidth=PageWidth-LeftMargin
	END IF

	vs_clip 1,LeftMargin,0,PageWidth,PageLength

	LastFont=vst_load_fonts+1			' Add 1 for default font
	FirstFont=2									' but start with the 1st loaded font

	vst_alignment 0,5							' align on top left of text

END SUB


'
' Clear screen or print current page and move on to next one
'
SUB NewPage
	SHARED Printer,LeftMargin,PageWidth,PageLength

	IF Printer THEN								' If printer output then ..
		v_updwk										' update to output page
	ELSE												' else screen output so ...
		pause											' wait for keypress or mouse
	END IF
	v_clrwk											' Clear screen or eject page

END SUB


'
' Wait for key or mouse button
'
SUB pause
	SHARED DUMMY
	DUMMY=evnt_multi(3,1,1,1, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0, 0,0,0,0)
END SUB




'
' Display or print font FontName$ at size P points.
' Vertical position if Y and cell height for font is H
' FontID (FontNum) is I
'
SUB showfont(FontName$,P,Y,H,I)
	SHARED LeftMargin,HSpacing
	v_gtext LeftMargin,Y,FontName$+" ("+LTRIM$(STR$(I))+")"+" at"+STR$(P)+" point"
	v_gtext LeftMargin+HSpacing,Y+H,"The quick brown fox jumps over lazy dogs!"
END SUB


'
' Unload fonts and close printer workstation if necessary
'
SUB ClearUp
	SHARED Printer,OldHandle

	vst_unload_fonts								' Unload the fonts

	IF Printer THEN								' If printer then ...
		v_clswk										' workstation must be closed
		CHANGE_HANDLE OldHandle					' restore original screen handle
	ELSE
		CLS
		WINDOW CLOSE 2
	END IF

	MOUSE 0

END SUB
